@@ -130,6 +130,7 @@ class ImportModel(QtCore.QAbstractTableModel):
 			existingRecords.append(query.value(0).toString())
 
 		recordsLoaded = set()
+
 		for record in set(records):
 			rec = ImportRecord(*record)
 
@@ -175,29 +176,37 @@ class ImportModel(QtCore.QAbstractTableModel):
 		if not item.isValid():
 			return QtCore.QVariant()
 
-		if role == QtCore.Qt.BackgroundColorRole:
-			if not self.__records[item.row()].valid:
-				return QtGui.QColor("#ffd3cf")
-			elif self.__records[item.row()].imported:
-				return QtGui.QColor("#b6ffac")
-			return QtCore.QVariant()
+		if role == QtCore.Qt.ForegroundRole:
+			if item.column() == 0:
+				if not self.__records[item.row()].valid:
+					return QtCore.QVariant(QtGui.QColor(255, 0, 0))
+				elif self.__records[item.row()].imported:
+					return QtCore.QVariant(QtGui.QColor(255, 165, 0))
+				return QtCore.QVariant(QtGui.QColor(0, 255, 0))
 
 		if role == QtCore.Qt.ToolTipRole:
-			return QtCore.QVariant(self.__records[item.row()].data)
+			return QtCore.QVariant(QtCore.QString.fromUtf8(self.__records[item.row()].data))
 
 		if role == QtCore.Qt.DisplayRole:
-			if item.column() == 0:
+			if item.column() == enum.kImportColumn_Status:
 				return QtCore.QVariant(self.__recordStatusToText(item))
-			elif item.column() == 1:
+
+			elif item.column() == enum.kImportColumn_Date:
 				return QtCore.QVariant(self.__records[item.row()].date)
-			elif item.column() == 2:
+
+			elif item.column() == enum.kImportColumn_TxDate:
 				return QtCore.QVariant(self.__records[item.row()].txdate)
-			elif item.column() == 3:
+
+			elif item.column() == enum.kImportColumn_Credit:
+				amount =  self.__records[item.row()].credit
+				return QtCore.QVariant('%.02f' % amount if amount else None)
+
+			elif item.column() == enum.kImportColumn_Debit:
+				amount =  self.__records[item.row()].debit
+				return QtCore.QVariant('%.02f' % abs(amount) if amount else None)
+
+			elif item.column() == enum.kImportColumn_Description:
 				return QtCore.QVariant(self.__records[item.row()].desc)
-			elif item.column() == 4:
-				return QtCore.QVariant(self.__records[item.row()].credit)
-			elif item.column() == 5:
-				return QtCore.QVariant(self.__records[item.row()].debit)
 
 		return QtCore.QVariant()
 
@@ -208,18 +217,18 @@ class ImportModel(QtCore.QAbstractTableModel):
 
 	def headerData (self, section, orientation, role):
 		if role == QtCore.Qt.DisplayRole:
-			if section == 0:
+			if section == enum.kImportColumn_Status:
 				return "Status"
-			elif section == 1:
+			elif section == enum.kImportColumn_Date:
 				return "Date"
-			elif section == 2:
+			elif section == enum.kImportColumn_TxDate:
 				return "TxDate"
-			elif section == 3:
-				return "Description"
-			elif section == 4:
+			elif section == enum.kImportColumn_Credit:
 				return "Credit"
-			elif section == 5:
+			elif section == enum.kImportColumn_Debit:
 				return "Debit"
+			elif section == enum.kImportColumn_Description:
+				return "Description"
 		return QtCore.QVariant()
 
 	def __recordStatusToText(self, index):
@@ -391,32 +400,55 @@ class RecordModel(QtSql.QSqlTableModel):
 
 		return super(RecordModel, self).data(item, role)
 
+	def toggleChecked(self, indexes):
+		checkedRecords = []
+		unCheckedRecords = []
+
+		for index in indexes:
+			recordId = self.index(index.row(), enum.kRecordColumn_RecordId).data().toPyObject()
+			if self.index(index.row(), enum.kRecordColumn_Checked).data(QtCore.Qt.CheckStateRole) == QtCore.Qt.Checked:
+				checkedRecords.append(recordId)
+			else:
+				unCheckedRecords.append(recordId)
+
+		with db.transaction():
+			if unCheckedRecords:
+				query = QtSql.QSqlQuery()
+				query.prepare("""
+					UPDATE records
+					   SET checked=1, checkdate=CURRENT_TIMESTAMP
+					 WHERE recordid IN (?)
+					""")
+				query.addBindValue(unCheckedRecords)
+				if not query.execBatch(QtSql.QSqlQuery.ValuesAsColumns):
+					raise Exception(query.lastError().text())
+	
+			if checkedRecords:
+				query = QtSql.QSqlQuery()
+				query.prepare("""
+					UPDATE records
+					   SET checked=0, checkdate=NULL
+					 WHERE recordid IN (?)
+					""")
+				query.addBindValue(checkedRecords)
+				if not query.execBatch(QtSql.QSqlQuery.ValuesAsColumns):
+					raise Exception(query.lastError().text())
+
+		self.select()
+
+		for index in indexes:
+			checkedIndex = self.index(index.row(), enum.kRecordColumn_Checked)
+			self.dataChanged.emit(checkedIndex, checkedIndex)
+
+	@utils.showWaitCursorDecorator
 	def setData(self, index, value, role=QtCore.Qt.EditRole):
 		""" Save new checkstate role changes in database
-		
-			Only send a single dataChanged signal for the whole row, not two separate
 		"""
-		status = False
 		if role == QtCore.Qt.CheckStateRole and index.column() == enum.kRecordColumn_Checked:
-			with utils.signalsBlocked(self):
-				if value.toPyObject() == QtCore.Qt.Checked:
-					status = (
-						super(RecordModel, self).setData(index, 1, QtCore.Qt.EditRole) and 
-						super(RecordModel, self).setData(self.index(index.row(), enum.kRecordColumn_CheckDate),
-														QtCore.QDateTime.currentDateTimeUtc(), 
-														QtCore.Qt.EditRole)
-					)
-				elif value.toPyObject() == QtCore.Qt.Unchecked:
-					status = (
-						super(RecordModel, self).setData(index, 0, QtCore.Qt.EditRole) and
-						super(RecordModel, self).setData(self.index(index.row(), enum.kRecordColumn_CheckDate),
-														QtCore.QVariant(), 
-														QtCore.Qt.EditRole)
-					)
-		if status:
-			self.dataChanged.emit(index, index)
+			self.toggleChecked([index])
+			return True
 
-		return status
+		return False
 
 	def headerData (self, section, orientation, role=QtCore.Qt.DisplayRole):
 		""" Set the header labels for the view
@@ -652,6 +684,7 @@ class TagProxyModel(QtGui.QSortFilterProxyModel):
 		return super(TagProxyModel, self).lessThan(left, right)
 
 class RecordProxyModel(QtGui.QSortFilterProxyModel):
+
 	# Signal emitted whenever there is a change to the filter
 	filterChanged = QtCore.pyqtSignal()
 
@@ -672,6 +705,7 @@ class RecordProxyModel(QtGui.QSortFilterProxyModel):
 		self._description = None
 		self._amountFilter = None
 		self._tagFilter = None
+		self._amountOperator = None
 
 	def clearFilters(self):
 		""" Clears all filters - does *not* call invalidate
@@ -727,7 +761,7 @@ class RecordProxyModel(QtGui.QSortFilterProxyModel):
 		if value != self._checked:
 			self._checked = value
 			self.invalidateFilter()
-	
+
 	def setCreditFilter(self, value):
 		""" Credit amount filter
 
@@ -760,6 +794,7 @@ class RecordProxyModel(QtGui.QSortFilterProxyModel):
 		""" Override invalidateFilter so that we can emit the filterChanged signal
 		"""
 		super(RecordProxyModel, self).invalidateFilter()
+		self.sort(self.sortColumn(), self.sortOrder())
 		self.filterChanged.emit()
 
 	def filterAcceptsRow(self, sourceRow, parent):
@@ -845,7 +880,7 @@ class RecordProxyModel(QtGui.QSortFilterProxyModel):
 				rightVal = self.sourceModel().index(right.row(), enum.kRecordColumn_RecordId).data().toPyObject()
 
 		if leftVal or rightVal:
-			return leftVal < rightVal
+			return leftVal > rightVal
 
 		return super(RecordProxyModel, self).lessThan(left, right)
 
@@ -857,8 +892,8 @@ class AccountEditModel(QtSql.QSqlTableModel):
 		if not item.isValid():
 			return QtCore.QVariant()
 
-		if role == QtCore.Qt.BackgroundColorRole and self.isDirty(item):
-				return QtCore.QVariant(QtGui.QColor(255,156,126))
+		if role == QtCore.Qt.ForegroundRole and self.isDirty(item):
+			return QtCore.QVariant(QtGui.QColor(255, 165, 0))
 
 		return super(AccountEditModel, self).data(item, role)
 
